/*******************************************************************************
 (c) 2005-2015 Copyright, Real-Time Innovations, Inc.  All rights reserved.
 RTI grants Licensee a license to use, modify, compile, and create derivative
 works of the Software.  Licensee has the right to distribute object form only
 for use with RTI products.  The Software is provided "as is", with no warranty
 of any type, including any warranty for fitness for any purpose. RTI is under
 no obligation to maintain or support the Software.  RTI shall not be liable for
 any incidental or consequential damages arising out of the use or inability to
 use the software.
 ******************************************************************************/

#include <iostream>
#include <dds/dds.hpp>
#include <rti/core/xtypes/DynamicDataTuples.hpp>
#include "Shapes.hpp"

using namespace dds::core;
using namespace dds::core::xtypes;
using namespace dds::domain;
using namespace dds::topic;
using namespace dds::pub;

void publisher_main(int domain_id, int sample_count)
{
    // Create the participant.
    DomainParticipant participant(domain_id);

    // changing this to 1024*1024 resolves the problem
    // dds::core::vector<uint8_t> longData(1024*1024);
    dds::core::vector<uint8_t> longData(1024*1024+1);
    ShapeType shape_data;
    shape_data.color(std::string("BLUE"));
    shape_data.id(1);
    shape_data.content(longData);

    Topic<ShapeType> topic(participant, "ShapeTopic");
    DataWriter<ShapeType> writer(Publisher(participant), topic);

    int id = 1;
    // Main loop
    for (int count = 0; (sample_count == 0) || (count < sample_count); ++count){
        shape_data.id(id++);
        writer.write(shape_data);

        std::cout << "sleeping" << std::endl;
        rti::util::sleep(Duration::from_millisecs(10000));
    }
}

int main(int argc, char* argv[])
{
    int domain_id = 0;
    int sample_count = 0; /* infinite loop */

    if (argc >= 2) {
        domain_id = atoi(argv[1]);
    }

    if (argc >= 3) {
        sample_count = atoi(argv[2]);
    }

    // To turn on additional logging, include <rti/config/Logger.hpp> and
    // uncomment the following line:
    // rti::config::Logger::instance().verbosity(rti::config::Verbosity::STATUS_ALL);

    try {
        publisher_main(domain_id, sample_count);
    } catch (std::exception ex) {
        std::cout << "Exception caught: " << ex.what() << std::endl;
        return -1;
    }

    return 0;
}
